package de.deversmann.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.*;
import java.util.List;

public class SpielfeldGUI extends JPanel {

    private JLabel[][] grid;
    private Set<String> errorCells = new HashSet<>();

    public SpielfeldGUI(int rows, int cols) {
        setLayout(new GridLayout(rows, cols));
        grid = new JLabel[rows][cols];

        for (int r = 0; r < rows; r++) {
            for (int c = 0; c < cols; c++) {
                JLabel label = createCellLabel("-", r, c);
                grid[r][c] = label;
                add(label);
            }
        }
    }

    public void updateGrid(int[][] feld) {
        if (feld == null) {
            return;
        }
        removeAll();
        int rows = feld.length;
        int cols = feld[0].length;
        setLayout(new GridLayout(rows, cols));
        grid = new JLabel[rows][cols];
        errorCells.clear();

        for (int r = 0; r < rows; r++) {
            for (int c = 0; c < cols; c++) {
                String text = String.valueOf(feld[r][c]);
                JLabel label = createCellLabel(text, r, c);
                grid[r][c] = label;
                add(label);
            }
        }
        revalidate();
        repaint();
    }

    public void setAllNonBlackToWhite() {
        if (grid == null) return;
        for (int r = 0; r < grid.length; r++) {
            for (int c = 0; c < grid[r].length; c++) {
                JLabel lbl = grid[r][c];
                if (lbl != null) {
                    if (!lbl.getBackground().equals(Color.BLACK)) {
                        lbl.setBackground(Color.WHITE);
                        lbl.setForeground(Color.BLACK);
                        clearErrorMark(r, c);
                    }
                }
            }
        }
    }

    public void markCellAsError(int row, int col) {
        if (!isValidCell(row, col)) return;
        errorCells.add(row + "," + col);
        grid[row][col].setBorder(BorderFactory.createLineBorder(Color.RED, 3));
    }

    public void clearErrorMark(int row, int col) {
        errorCells.remove(row + "," + col);
        if (!isValidCell(row, col)) return;
        grid[row][col].setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY, 1));
    }

    public List<int[]> getBlackCells() {
        List<int[]> black = new ArrayList<>();
        if (grid == null) return black;

        for (int r = 0; r < grid.length; r++) {
            for (int c = 0; c < grid[r].length; c++) {
                JLabel lbl = grid[r][c];
                if (lbl != null && lbl.getBackground().equals(Color.BLACK)) {
                    black.add(new int[]{r, c});
                }
            }
        }
        return black;
    }

    public void setCellBlack(int row, int col) {
        if (!isValidCell(row, col)) return;
        JLabel lbl = grid[row][col];
        lbl.setBackground(Color.BLACK);
        lbl.setForeground(Color.WHITE);
        clearErrorMark(row, col);
    }

    private JLabel createCellLabel(String text, int row, int col) {
        JLabel label = new JLabel(text, SwingConstants.CENTER);
        label.setOpaque(true);
        label.setBackground(Color.GRAY);
        label.setForeground(Color.BLACK);
        label.setFont(new Font("Arial", Font.BOLD, 24));
        label.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY, 1));

        label.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                toggleColor(label, row, col);
            }
        });
        return label;
    }

    private void toggleColor(JLabel label, int row, int col) {
        if (label.getBackground().equals(Color.GRAY) || label.getBackground().equals(Color.WHITE)) {
            label.setBackground(Color.BLACK);
            label.setForeground(Color.WHITE);
        } else {
            label.setBackground(Color.WHITE);
            label.setForeground(Color.BLACK);
        }
        if (errorCells.contains(row + "," + col)) {
            clearErrorMark(row, col);
        }
    }

    private boolean isValidCell(int r, int c) {
        if (grid == null) return false;
        return (r >= 0 && r < grid.length && c >= 0 && c < grid[r].length);
    }
}
